@@ -10,7 +10,7 @@ my $StartingOf = {
     'rfc822'  => ['Content-type: message/rfc822'],
 };
 
-sub headerlist  { return ['X-AHMAILID'] }
+sub headerlist  { return ['x-ahmailid'] }
 sub description { 'TransWARE Active!hunter' };
 sub scan {
     # Detect an error from TransWARE Active!hunter
@@ -34,7 +34,6 @@ sub scan {
     return undef unless defined $mhead->{'x-ahmailid'};
 
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my @hasdivided = split("\n", $$mbody);
     my $rfc822part = '';    # (String) message/rfc822-headers part
     my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
     my $blanklines = 0;     # (Integer) The number of blank lines
@@ -42,7 +41,7 @@ sub scan {
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
 
-    for my $e ( @hasdivided ) {
+    for my $e ( split("\n", $$mbody) ) {
         # Read each line between the start of the message and the start of rfc822 part.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
@@ -61,16 +60,15 @@ sub scan {
         }
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # After "message/rfc822"
+            # Inside of the original message part
             unless( length $e ) {
-                $blanklines++;
-                last if $blanklines > 1;
+                last if ++$blanklines > 1;
                 next;
             }
             push @$rfc822list, $e;
 
         } else {
-            # Before "message/rfc822"
+            # Error message part
             next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
@@ -99,12 +97,11 @@ sub scan {
                 next if length $v->{'diagnosis'};
                 $v->{'diagnosis'} ||= $e;
             }
-        } # End of if: rfc822
+        } # End of error message part
     }
-
     return undef unless $recipients;
-    require Sisimai::String;
 
+    require Sisimai::String;
     for my $e ( @$dscontents ) {
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
         $e->{'agent'}     = __PACKAGE__->smtpagent;
