@@ -103,7 +103,7 @@ my $StateTable = {
     '18' => { 'command' => 'DATA', 'reason' => 'filtered' },
 };
 
-sub headerlist  { return ['X-Failed-Recipients'] }
+sub headerlist  { return ['x-failed-recipients'] }
 sub description { 'Google Gmail: https://mail.google.com' }
 sub scan {
     # Detect an error from Google Gmail
@@ -172,9 +172,7 @@ sub scan {
     return undef unless rindex($mhead->{'from'}, '<mailer-daemon@googlemail.com>') > -1;
     return undef unless index($mhead->{'subject'}, 'Delivery Status Notification') > -1;
 
-    require Sisimai::Address;
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my @hasdivided = split("\n", $$mbody);
     my $rfc822part = '';    # (String) message/rfc822-headers part
     my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
     my $blanklines = 0;     # (Integer) The number of blank lines
@@ -183,7 +181,7 @@ sub scan {
     my $statecode0 = 0;     # (Integer) The value of (state *) in the error message
     my $v = undef;
 
-    for my $e ( @hasdivided ) {
+    for my $e ( split("\n", $$mbody) ) {
         # Read each line between the start of the message and the start of rfc822 part.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
@@ -199,16 +197,15 @@ sub scan {
         }
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # After "message/rfc822"
+            # Inside of the original message part
             unless( length $e ) {
-                $blanklines++;
-                last if $blanklines > 1;
+                last if ++$blanklines > 1;
                 next;
             }
             push @$rfc822list, $e;
 
         } else {
-            # Before "message/rfc822"
+            # Error message part
             next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
@@ -231,26 +228,24 @@ sub scan {
 
             if( $e =~ /\A[ \t]+([^ ]+[@][^ ]+)\z/ ) {
                 # kijitora@example.jp: 550 5.2.2 <kijitora@example>... Mailbox Full
-                if( length $v->{'recipient'} ) {
+                if( $v->{'recipient'} ) {
                     # There are multiple recipient addresses in the message body.
                     push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                     $v = $dscontents->[-1];
                 }
 
-                my $addr0 = Sisimai::Address->s3s4($1);
-                if( Sisimai::RFC5322->is_emailaddress($addr0) ) {
-                    $v->{'recipient'} = $addr0;
-                    $recipients++;
-                }
+                my $r = Sisimai::Address->s3s4($1);
+                next unless Sisimai::RFC5322->is_emailaddress($r);
+                $v->{'recipient'} = $r;
+                $recipients++;
+
             } else {
                 $v->{'diagnosis'} .= $e.' ';
             }
-        } # End of if: rfc822
+        } # End of error message part
     }
     return undef unless $recipients;
 
-    require Sisimai::String;
-    require Sisimai::SMTP::Status;
     for my $e ( @$dscontents ) {
         $e->{'agent'}     = __PACKAGE__->smtpagent;
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
@@ -289,12 +284,10 @@ sub scan {
         }
         next unless $e->{'reason'};
 
-        # Set pseudo status code
-        $e->{'status'} = Sisimai::SMTP::Status->find($e->{'diagnosis'});
-        if( $e->{'status'} =~ /\A[45][.][1-7][.][1-9]\z/ ) {
-            # Override bounce reason 
-            $e->{'reason'} = Sisimai::SMTP::Status->name($e->{'status'});
-        }
+        # Set pseudo status code and override bounce reason 
+        $e->{'status'} = Sisimai::SMTP::Status->find($e->{'diagnosis'}) || '';
+        next unless $e->{'status'} =~ /\A[45][.][1-7][.][1-9]\z/;
+        $e->{'reason'} = Sisimai::SMTP::Status->name($e->{'status'}) || '';
     }
 
     $rfc822part = Sisimai::RFC5322->weedout($rfc822list);
