@@ -18,7 +18,7 @@ sub match {
          access[ ]denied[.][ ]ip[ ]name[ ]lookup[ ]failed
         |access[ ]from[ ]ip[ ]address[ ].+[ ]blocked
         |all[ ]mail[ ]servers[ ]must[ ]have[ ]a[ ]ptr[ ]record[ ]with[ ]a[ ]valid[ ]reverse[ ]dns[ ]entry
-        |bad[ ]sender[ ]ip[ ]address
+        |bad[ ](:?dns[ ]ptr[ ]resource[ ]record|sender[ ]ip[ ]address)
         |banned[ ]sending[ ]ip  # Office365
         |blacklisted[ ]by
         |(?:blocked|refused)[ ]-[ ]see[ ]https?://
