@@ -13,8 +13,8 @@ sub match {
     # @since v4.0.0
     my $class = shift;
     my $argv1 = shift // return undef;
-    my $index = ['message too large'];
 
+    state $index = ['message too large'];
     return 1 if grep { rindex($argv1, $_) > -1 } @$index;
     return 0;
 }
@@ -35,7 +35,7 @@ sub true {
     # Delivery status code points "exceedlimit".
     # Status: 5.2.3
     # Diagnostic-Code: SMTP; 552 5.2.3 Message size exceeds fixed maximum message size
-    return 1 if Sisimai::SMTP::Status->name($argvs->deliverystatus) eq 'exceedlimit';
+    return 1 if (Sisimai::SMTP::Status->name($argvs->deliverystatus) || '') eq 'exceedlimit';
 
     # Check the value of Diagnosic-Code: header with patterns
     return 1 if __PACKAGE__->match(lc $argvs->diagnosticcode);
