@@ -7,6 +7,7 @@ sub list {
         Bite.pm
             Bite/Email.pm
                 Bite/Email/Activehunter.pm
+                Bite/Email/Amavis.pm
                 Bite/Email/AmazonSES.pm
                 Bite/Email/AmazonWorkMail.pm
                 Bite/Email/Aol.pm
@@ -107,6 +108,7 @@ sub list {
             Reason/UserUnknown.pm
             Reason/Vacation.pm
             Reason/VirusDetected.pm
+        RFC1894.pm
         RFC3464.pm
         RFC3834.pm
         RFC5322.pm
@@ -116,6 +118,7 @@ sub list {
             Rhost/GoDaddy.pm
             Rhost/FrancePTT.pm
             Rhost/KDDI.pm
+            Rhost/TencentQQ.pm
         SMTP.pm
             SMTP/Error.pm
             SMTP/Reply.pm
